#!/usr/bin/perl

use strict;
#use warnings;

use File::Find;
use File::Spec::Functions;

print "\nTestPerl.pl  1.02  2022-04-11\n";
print "Tests to see if you are set up to process Frogscript with MarkdownWrapper.pl\n";

my $ConfigDir = "etc/markcms";	# Where we look for the config files.
my $TestCFile = "frog.cfg";		# File to look for in the config directory.
my $TestCFileSite = "frog-site.cfg"; # Site-specfic file to look for
									# It will *not* be there after the initial install...
									# The user is supposed to edit it and copy it to the config dir.

my $s = "";
my @WorkLog;
my @ErrLog;          # Collects information about fatal errors and warnings
my $fatal="0";       # Fatal error counter
my $good="0";
my $SiteConfig="";
# Collect all the Perl modules that are installed.
# Derived from http://support.godaddy.com/help/article/1488/can-i-add-perl-modules-to-my-hosting-account
find { wanted => sub { $s .= canonpath $_ . "\n" if /\.pm\z/ }, no_chdir => 1 }, @INC;

if (6 ==9) {
	# At this point $s contains fully qualified paths for all the Perl modules that are installed.
	# We just have to tuck this away now that we have it.
	writefile_byref("InstalledPerlModules.txt", \$s);
	#$s = ""; readfile_byref("InstalledPerlModules.txt", \$s);
}

# Change directory separators to be "::".  Which is Perlspeak for "directory separater."
$s =~ s|\\|/|sg;
$s =~ s|/|::|sg;

# Make Perl wade through the entire list to find what we want to find.
my @modules = ("FrogUtils::utility", "FrogUtils::MarkdownW.pm", "FrogUtils::SmartyPantsW", "File::Path", "Digest::MD5", "Date::Calc");
my $match;
for $match (@modules) {
	if ( $s =~ m/$match/ ) {
		push (@WorkLog, "  Module '$match' is loaded.\n");
		$good++;
	} else {
		push (@ErrLog, "  Uh, oh.  Module '$match' does not seem to be installed.\n");
		$fatal++;
	}
}

my $HomePath = Check4Home();
my $HomeVar = "";
if ($good) { push(@WorkLog, "\n") }
if ( $^O =~ /MSWin.*/i ) { 
	$HomeVar = "FHOME"; 
	$SiteConfig = "etc/markcms-windows";
} else {
	$HomeVar = "HOME";
	$SiteConfig = "etc/markcms-linux";
}
if ($HomePath) {
	push (@WorkLog, "  Your $HomeVar directory is '$HomePath'\n");
	my $CDir = prep_path("$HomePath$ConfigDir");
	if ( -d $CDir ) {
		push (@WorkLog, "  Your configuration directory ($CDir) exists.\n");
		$good++;
		if ( -f $CDir . $TestCFile ) {
			push (@WorkLog, "  '$TestCFile' exists in your configuration directory.\n");
			$good++;
		} else {
			push(@ErrLog, "  Uh, oh.  '$TestCFile' doesn't exist in your configuration directory.\n");
			$fatal++;
		}
		if ( -f $CDir . $TestCFileSite ) {
			push (@WorkLog, "  '$TestCFileSite' exists in your configuration directory.\n");
			$good++;
		} else {
			push(@ErrLog, "  Uh, oh.  '$TestCFileSite' doesn't exist in your configuration directory.\n");
			push(@ErrLog, "      You were supposed to edit them in $SiteConfig and copy them to $CDir.\n");
			$fatal++;
		}
	} else {
		push (@ErrLog, "  Uh, oh. Your configuration directory ($CDir) does not exist.\n");
		$fatal++;
	}	
} else {
	push (@ErrLog, "  Uh, oh.  Your $HomeVar environment variables isn't set.\n");
	$fatal++;
}

print "\nThis is good:\n";
foreach $s (@WorkLog) {
   print $s;
}
if ($fatal) {
	print "\nThis is not good, you have $fatal errors:\n";
	foreach $s (@ErrLog) {
		print $s;
	}
	my $s = "";
	print "\nPerl modules are installed in:\n";
	foreach $s (@INC) {
		$s = prep_path($s);
		print "  $s\n";
	}	
	print "\nCheck to see that your 'PERL5LIB' environment variable is set correctly.\n";
	print "(Look at the install docs for anything that mentions 'PERL5LIB'.)\n";
} else {
	print "\n";
}

# Test to see if one of the unxutils programs is on the path.
# Only need to test this on Windows.
my @utils;
push (@utils, "cp.exe");
#push (@utils, "less.exe");
push (@utils, "ls.exe");
push (@utils, "touch.exe");

if ( $^O =~ /MSWin.*/i ) {	
	my $unx = "";
	my $unxcnt = 0;
	print "Testing for several unxutil programs we need...\n";
	foreach $unx (@utils) {
		my $Command = "$unx --version";
		#print "Trying unxutils program '$Command'. It returned\n";  
		my $result = `$Command`;
		chomp $result;
		#if ($result) { print "\n$result\n" }
		if ($result) {
			#print "Result: '$result'\nWhich is good.\n";
			$good++;
			$unxcnt++;
		} else {
			#print "\n  Uh, oh.  unxutils doesn't seem to be on the path.\n";
			$fatal++;
		}
	}
	if ($unxcnt) {
		print "Good.  $unxcnt out of 3 unxutils programs were found.\n";
	} else {
		print "Not so good.  The unxutil programs weren't found on the path.\n";
		print "Get them from http://unxutils.sourceforge.net/\n";
		print "And make sure they are on the path\n";
	}
}

print "\nTesting to see if ImageMagick is installed\n";
my $MagicCommand = "";
if ( $^O =~ /MSWin.*/i ) {
	my $Command = "convert /?";
	my $result = `$Command`;
	if ( $result =~ m/Converts a FAT volume to NTFS/is) {
		print "ImageMagick not installed!";
		print "\n'convert /?' returns 'Converts a FAT volume to NTFS\n'";
		print "Which could mean a really bad day.\n";
		print "(Older versions of ImageMagick used 'convert' instead of 'magick'.)\n";
		$fatal++;
	}
}
my $Command = "magick --version";
my $result = `$Command`;
if ( $result =~ m/Version\: ImageMagick/is) {
	print "Good! ImageMagick (magick) is installed\n";
	$MagicCommand = "magick";
} else {
	$Command = "convert --version";
	$result = `$Command`;
	if ( $result =~ m/Version\: ImageMagick/is) {
		print "Good! ImageMagick (convert) is installed. Which is close enough.\n";
		$MagicCommand = "convert";
	} else {
		print "ImageMagick is not installed.\n";
		print "Get ImageMagick from 'https://imagemagick.org/index.php'\n";
		$fatal++;
	}
}	# End of ImageMagick test.

if ($fatal) {
	print "\n!!! There were $fatal fatal errors, which means that Frogdown (MarkdownWrapper.pl)\nprobably won't run.\n";
}

exit 0;

# $HomePath = Check4Home();
sub Check4Home {
	my $HomePath = "";
	if ( $^O =~ /MSWin.*/i ) {
		if ( $ENV{'FHOME'} ) {	# i.e., SET should contain 'FHOME=C:\home'
			$HomePath = $ENV{'FHOME'};
		} elsif ( $ENV{'WHOME'} ) {	# Alternate
			$HomePath = $ENV{'WHOME'};
		} elsif ( -d "C:/Home" ) {	# Default install dir.  (Per instructions.)
		$HomePath = "C:/Home/";
		} else {
			print "Environment variable 'FHOME' does not exist.\n";
			return "";
		}
	} else {
	    if ( $ENV{'FHOME'} ) {	# Sometimes we need to leave HOME.
            $HomePath = $ENV{'FHOME'};
            print "'FHOME' is used in place of 'HOME'\n";
		} elsif ( $ENV{'HOME'} ) {
			$HomePath = $ENV{'HOME'};
		} else {
			print "Environment variable 'HOME' does not exist.  We need it.\n";
			return "";
		}
	}
	$HomePath  =~ s|\\|/|g;
	$HomePath = add_rslash($HomePath);
	return $HomePath;
}

# Taken from frogutils::utility. Which we can't load because we are testing
# to see if frogutils::utility is installed.

# Add trailing slash.  But *not* if it is "/" or ""
# Designed for preparing base paths to be joined with directories.
# So $base . $relpath will do what you expect if $base is "", or "/"
sub add_rslash {
	my $s = shift;
	$s =~ s|\\|/|g;	# New
	$s = trim($s);
	unless ( $s ) { return $s; }
	if ( $s eq "/" ) { return $s; }
	$s = strip_rslash($s) . "/";
	return $s;	# Added 2012-01-27!!!!
}

# Add slash at the beginning (if it needs it.)
sub add_lslash {
	my $s = shift;
	$s = trim($s);
	unless ( $s ) { return $s; }
	if ( $s eq "/" ) { return $s; }
	#if ( ! /^\// ) { $s = '/' . $s; }
	unless ( $s =~ /^\// ) { $s = '/' . $s; }
	return $s;
}

# Removes any trailing backslashes.
sub strip_rbslashxx {
	my $s = shift;
	$s = remove_slashes($s);
	$s =~ s|(.*)\\$|\1|;
	return $s;
}

# Removes any trailing slash
sub strip_rslash {
	my $s = shift;
	$s = remove_slashes($s);
	$s =~ s|(.*)/$|\1|;
	return $s;
}

# Removes doubled slashes (except for leading ones, to protect network path.)
sub remove_slashes {
	my $s = shift;
	
	$s =~ s|\\|/|g;	# New
	my $netflag = "";
	if ($s =~ m|(^//.*?)/| ) {
		#print "Network path '$1' in $s\n"; #exit -1;
		$netflag = "-1";
	}
	while ( $s =~ m|//| ) {
		$s =~ s|//|/|g;
	}
	if ($netflag) {
		$s = "/" . $s;
		#print "Network path.  Returning '$s'\n";
	}
	return $s;
}

# Remove leading and trailing whitespace
sub trim {
	my $s = shift;
	$s =~ s/^\s*//;	# Remove whitespace from the left
	$s =~ s/\s+$//;	# Remove whitespace from the right
	return $s;

}

sub prep_path {
	my $s = shift;
	if ($s eq "." || $s eq "..") { return $s}
	$s =~ s|\n||g;	# Remove '\n's
	$s =~ s|\\|/|g;	# Convert '\'s to '/'s
	unless($s =~ m/:/ ) { $s = add_lslash($s); }
	$s = add_rslash($s);	# Add a trailing '/'
	return $s;
}

sub writefile {
	my $Filename = shift;		# Filename
	my $FileContents = shift;	# Contents of file
	
	open (WRITEFILE, ">$Filename") or die "Could not open '$Filename' $!";
	print WRITEFILE $FileContents;
	close WRITEFILE;
	return 1;
}

# readfile_byref ($filein, \$cfile);
sub readfile_byref {
	my $Filename = shift;			# Regular scalar
	my $FileContentsRef = shift;	# Reference to a scalar
	
	if (-f $Filename ) {
		print "Reading '$Filename'\n";
		open (READFILE, $Filename) or die "Could not open '$Filename' $!";
		${$FileContentsRef} = join('', <READFILE>);
		${$FileContentsRef} =~ s{\r\n}{\n}g; # DOS to Unix
		${$FileContentsRef} =~ s{\r}{\n}g; 	# Mac to Unix
		close READFILE;
		return 1;
	} else {
		print "'$Filename' not found.\n";
		#${$FileContentsRef} = "";
		return 0;
		#exit -1;
		#return "";
	}
}

# writefile_byref ($filein, \$cfile);
sub writefile_byref {
	my $Filename = shift;			# Regular scalar
	my $FileContentsRef = shift;	# Reference to a scalar
	
	open (WRITEFILE, ">$Filename") or die "Could not open '$Filename' $!";
	print WRITEFILE ${$FileContentsRef};
	close WRITEFILE;
	return 1;
}

sub readfile {
	my $filein = shift;
	
	if ( -f $filein ) {	# Read into a string
		open (READFILE, $filein);
		my $cfile = join('', <READFILE>);	# Assumes that 'undef $/;' has *not* beeen done.
		close READFILE;
		CleanNewLines($cfile);
		return $cfile;
	} else {
		return "";
	}
}	

sub CleanNewLines {
	my $s = shift;
	$s =~ s{\r\n}{\n}g; # DOS to Unix
	$s =~ s{\r}{\n}g; 	# Mac to Unix
	return $s;
}