#!/usr/bin/perl

# Misc. utility functions

package FrogUtils::utility;

use Date::Calc qw( Add_Delta_DHMS );
use POSIX;
#use Math::Trig qw(great_circle_distance deg2rad acos);
#use strict;
use 5.006;
#use warnings;

# Not sure about the next line.
our (@ISA, @EXPORT);
# Next two lines are boilerplate (and are mandatory.)
require Exporter;
@ISA = Exporter;
# We need to qw() each subroutine name if we want to use them transparently.
# (If not exported, can be called like 'wsub::hello()'.)
@EXPORT = qw/ 
				all_back all_back2 remove_doubles trim ltrim rtrim 
				is_comment is_linux
				remove_all_slashes remove_slashes strip_rslash remove_bslashes
				strip_rbslash add_rslash add_lslash strip_lslash
				prep_path return_path return_file return_fname return_fext
				commify to2 to3 get_now readfile readfile_byref writefile writefile_byref format_distance
				get_arg welhello2
				Parmify_byref
			/;
# Moved Deg2Dec calc_distance_rad_hav2 calc_distance_dec

# Start of subs
# -----------------------------------------
#

use strict;

# Just to prove that the module exists and works.
sub welhello2 {
	my $s = shift;
	return $s;
}


sub all_back {
	my $s = shift;
	#$s =~ s|\\|/|g;
	#$s = remove_slashes($s);
	$s =~ s|/|\\|g;
	return $s;
}

# (Effectively) changes forward slashes to backslashes.  Then removes doubled slashes.
sub all_back2 {
	my $s = shift;
	$s =~ s|\\|/|g;
	$s = remove_slashes($s);
	$s =~ s|/|\\|g;
	return $s;
}

sub remove_doubles {
	my $s = shift;
	$s =~ s|\\|/|g;
	$s = remove_slashes($s);
	return $s;		
}

# Remove leading and trailing whitespace
sub trim {
	my $s = shift;
	$s =~ s/^\s*//;	# Remove whitespace from the left
	$s =~ s/\s+$//;	# Remove whitespace from the right
	return $s;

}

# Remove leading whitespace
sub ltrim {
	my $s = shift;
	$s =~ s/^\s*//;	# Remove whitespace from the left
	return $s;

}

# Remove trailing whitespace
sub rtrim {
	my $s = shift;
	$s =~ s/\s+$//;	# Remove whitespace from the left
	return $s;
}

# Is the string a comment line?
sub is_comment {
	my $s = shift;
	chomp ($s);
	$s = trim($s);
	if ($s =~ /^#/ || !$s) {
		#print "True\n";
		return "-1";
	} else {
		#print "False\n";
		return "";
	}
}

# Are we running under Linux?  (Doesn't work!)
sub is_linux {
	#print "Testing |$^O| \n";
	if ($^O =~ /linix/ ) {	# 'Programming Perl' p. 137
		return "-1";
	} else {
		return "";
	}
}

# Removes all doubled slashes (presumed to not contain a network path.)
sub remove_all_slashes {
	my $s = shift;
	
	$s =~ s|\\|/|g;	# New
	while ( $s =~ m|//| ) {
		$s =~ s|//|/|g;
	}
	return $s;
}

# Removes doubled slashes (except for leading ones, to protect network path.)
sub remove_slashes {
	my $s = shift;
	
	$s =~ s|\\|/|g;	# New
	my $netflag = "";
	if ($s =~ m|(^//.*?)/| ) {
		#print "Network path '$1' in $s\n"; #exit -1;
		$netflag = "-1";
	}
	while ( $s =~ m|//| ) {
		$s =~ s|//|/|g;
	}
	if ($netflag) {
		$s = "/" . $s;
		#print "Network path.  Returning '$s'\n";
	}
	return $s;
}

# Removes any trailing slash
sub strip_rslash {
	my $s = shift;
	$s = remove_slashes($s);
	$s =~ s|(.*)/$|\1|;
	return $s;
}

# Removes doubled back slashes
sub remove_bslashes {
	my $s = shift;
	while ( $s =~ s|\\\\|\\|g) {}
	return $s;
}

# Removes any trailing backslashes.
sub strip_rbslash {
	my $s = shift;
	$s = remove_slashes($s);
	$s =~ s|(.*)\\$|\1|;
	return $s;
}

# Add trailing slash.  But *not* if it is "/" or ""
# Designed for preparing base paths to be joined with directories.
# So $base . $relpath will do what you expect if $base is "", or "/"
sub add_rslash {
	my $s = shift;
	$s =~ s|\\|/|g;	# New
	$s = trim($s);
	unless ( $s ) { return $s; }
	if ( $s eq "/" ) { return $s; }
	$s = strip_rslash($s) . "/";
	return $s;	# Added 2012-01-27!!!!
}

# Add slash at the beginning (if it needs it.)
sub add_lslash {
	my $s = shift;
	$s = trim($s);
	unless ( $s ) { return $s; }
	if ( $s eq "/" ) { return $s; }
	#if ( ! /^\// ) { $s = '/' . $s; }
	unless ( $s =~ /^\// ) { $s = '/' . $s; }
	return $s;
}

# Removes any leading slash
sub strip_lslash {
	my $s = shift;
	$s = remove_slashes($s);
	$s =~ s|^/(.*)|\1|;
	return $s;
}

sub prep_path {
	my $s = shift;
	if ($s eq "." || $s eq "..") { return $s}
	$s =~ s|\n||g;	# Remove '\n's
	$s =~ s|\\|/|g;	# Convert '\'s to '/'s
	unless($s =~ m/:/ ) { $s = add_lslash($s); }
	$s = add_rslash($s);	# Add a trailing '/'
	return $s;
}

# Returns the path component.  i.e., everything before the final "/"
sub return_path {
	my $s = shift;
	$s = trim($s);
	unless ( $s =~ m|/| ) { return ""; } 
	$s =~ /(.*)\/([^\/]*$)/;
	return $1;
	#return remove_slashes($1);
}

# Returns the file component. i.e. everything after the final "/"
sub return_file {
	my $s = shift;
	$s = trim($s);
	unless ( $s =~ m|/| ) { return $s; }
	$s =~ /(.*)\/([^\/]*$)/;
	return $2;
}

# Returns the base name.  i.e., everything before the final "."
sub return_fname {
	my $s = shift;
	$s = trim($s);
	unless ($s =~ m/\./) { return $s } # If there is no '.'
	$s =~ /(.*)\.([^\.]*$)/;
	return $1;
}

# Returns the extension.  i.e.  everything after the final "."
sub return_fext {
	my $s = shift;
	$s = trim($s);
	$s =~ /(.*)\.([^\.]*$)/;
	return $2;
}

# Format a number to have commas in it.
sub commify {
# http://www.perl.com/doc/FAQs/FAQ/oldfaq-html/Q4.34.html
    local($_) = shift;
    1 while s/^(-?\d+)(\d{3})/$1,$2/;
    return $_;
} 

sub to2 {
		my $x = shift;
		my $s;
		$s = sprintf("%.2f",$x);
		$s = commify($s);
		return $s;
}

sub to3 {
		my $x = shift;
		my $s;
		$s = sprintf("%.3f",$x);
		$s = commify($s);
		return $s;
}

# Return current YYYY-MM-DD HH:MM:SS
sub get_now {
	my @timeData = localtime(time);
	my ($s, $m, $h, $d, $M, $y) = @timeData;
	$y += 1900;
	$M++;
	return sprintf("%04u-%02u-%02u %02u:%02u:%02u",$y, $M, $d, $h, $m, $s);
}

sub format_distance {
	my $distance = shift;
	if ($distance < 1 && 0 ) {
		my $feet = $distance * 5280;
		return sprintf("%.1f feet", $feet);
	} else {
		return sprintf("%.4f miles", $distance);
	}
}

# my $FileContents = readfile($FileName);	
sub readfile {
	my $filein = shift;
	
	if ( -f $filein ) {	# Read into a string
		open (READFILE, $filein);
		my $cfile = join('', <READFILE>);	# Assumes that 'undef $/;' has *not* beeen done.
		close READFILE;
		$cfile =~ s{\r\n}{\n}g; # DOS to Unix
		$cfile =~ s{\r}{\n}g; 	# Mac to Unix
		return $cfile;
	} else {
		return "";
	}
}	

sub CleanNewLines {
	my $s = shift;
	$s =~ s{\r\n}{\n}g; # DOS to Unix
	$s =~ s{\r}{\n}g; 	# Mac to Unix
	return $s;
}

# readfile_byref ($FileName, \$FileContents);
sub readfile_byref {
	my $Filename = shift;			# Regular scalar
	my $FileContentsRef = shift;	# Reference to a scalar
	
	if (-f $Filename ) {
		#print "Reading '$Filename'\n";
		open (READFILE, $Filename) or die "Could not open '$Filename' $!";
		${$FileContentsRef} = join('', <READFILE>);
		${$FileContentsRef} = s{\r\n}{\n}g; # DOS to Unix
		${$FileContentsRef} = s{\r}{\n}g; 	# Mac to Unix
		close READFILE;
		return 1;
	} else {
		print "'$Filename' not found.\n";
		return "";
	}
}

# writefile($FileName, $FileContents);
sub writefile {
	my $Filename = shift;		# Filename
	my $FileContents = shift;	# Contents of file
	
	open (WRITEFILE, ">$Filename") or die "Could not open '$Filename' $!";
	print WRITEFILE $FileContents;
	close WRITEFILE;
	return 1;
}

# writefile_byref ($filein, \$cfile);
sub writefile_byref {
	my $Filename = shift;			# Regular scalar
	my $FileContentsRef = shift;	# Reference to a scalar
	
	open (WRITEFILE, ">$Filename") or die "Could not open '$Filename' $!";
	print WRITEFILE ${$FileContentsRef};
	close WRITEFILE;
	return 1;
}

# How references work.  Example is to declare a scalar and an array, and assign values to each (i.e. fill some array elements)
# Call like 'reference_demo(\$scalar, \@array)'  The scalar will be printed.  The array will be modified.
sub reference_demo {
	my $ScalarRef = shift;	# Reference to a scalar
	my $ArrayRef = shift;	# Reference to an array
	
	print "Scalar deferenced: ${$ScalarRef}\n";		# Dereferencing the scalar ('$$ScalarRef' works, but is discouraged.)
	my $line;										# Regular scalar variable
	
	my $i = 0;
	foreach $line (@{$ArrayRef}) {	# Dereferencing the array in array context ('@$arrayref' works, but is discouraged.)
		print "$line\t";
		$ArrayRef->[$i] = "Here is \$i: $i";	# Dereferencing the array in scalar context (and modifying the array elements)
		print "$ArrayRef->[$i]\n";				# Dereferencing the array in scalar context
		++$i;
	}
}

# Will parse "mft=3|azf=99|swert|foo=bar" into key-value pairs
# into the hash that is passed by reference
# '=' in value side needs to be escaped like '\\='
# my $s = "mft=3|azf=99|swert|foo=bar|open-root=free\\=\\=beer|root=free\\=beer";
# my %hash;
# Parmify_byref($s, \%hash);
# %hash can be used normally (it does not need referencey syntax.)
# Note that keys are lower cased.
sub Parmify_byref {
	my $s = shift;
	my $ParmsRef = shift;
	
	for (keys %$ParmsRef) { delete $ParmsRef->{$_}; }	# Empty out the %ParmsRef hash before we start.
	$s = trim($s);
	while ($s =~ s/\\=/%%%EQUAL%%%/g) {}
	my @l = split(/\|/, $s);
	my ($AParm, $key, $value);
	foreach $AParm (@l) {
		#print " $AParm\n";
		($key, $value) = split(/=/, $AParm);
		$key = lc(trim($key)); $value = trim($value);
		if ($key) { 
			$value =~ s/%%%EQUAL%%%/=/g;
			$ParmsRef->{$key} = $value; 
		}
	}
	if (6 == 9) {		# Debugging....
		foreach $key (sort(keys %$ParmsRef)) {
			$value = $ParmsRef->{$key};
			print " '$key' = '$value'\n";
		}
	}
}

# Changes to the directory that is contained in a full pathname (as would be passed when a file is clicked on)
# i.e. if $arg is "C:\home\public_html\index.html", after calling get_arg() , the current directory would be
# C:\home\public_html\  
# Returns true (1) if successful, false (0) if unsuccessful)
# *Sometimes* the pwd is 'C:\Windows\System' after clicking on a file, so you can't assume that the pwd is pwd!!!!
# get_arg() forces pwd to be the directory that is implied by the argument's path.
sub get_arg {
	my $arg = shift;
	if (-f $arg) {
		$arg =~ s|\\|/|g;
		my $pwd = return_path($arg);
		if ( -d $pwd) {
			chdir($pwd);
			return 1;
		} else {
			return 0;
		}
	}
}



#
# -----------------------------------------
# End of subs.  Don't change the next line.
1;
